package com.sf.vertx.utils;

import static com.sf.vertx.constans.SACConstants.API_CONFIG;
import static com.sf.vertx.constans.SACConstants.API_SERVICE_TYPE;
import static com.sf.vertx.constans.SACConstants.APP_CONFIG;
import static com.sf.vertx.constans.SACConstants.GATEWAY_BODY_CODE;

import java.math.BigDecimal;
import java.util.List;

import org.apache.commons.lang3.StringUtils;

import com.sf.vertx.api.pojo.ApiConfig;
import com.sf.vertx.api.pojo.AppConfig;
import com.sf.vertx.api.pojo.MockExpectation;
import com.sf.vertx.api.pojo.MockMatchCondition;
import com.sf.vertx.api.pojo.MockResponse;
import com.sf.vertx.enums.GatewayError;
import com.sf.vertx.enums.GatewayServiceType;
import com.sf.vertx.enums.MatchType;
import com.sf.vertx.exception.ServiceException;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.NumberUtil;
import io.vertx.core.json.JsonObject;
import io.vertx.ext.web.RoutingContext;
import lombok.experimental.UtilityClass;
import lombok.extern.slf4j.Slf4j;

/**
 * 功能描述:
 *
 * @author a_kun
 * @date 2024/5/24 12:30
 */
@Slf4j
@UtilityClass
public class AppUtils {
	
	/**
	 * 将AppConfig设置到上下文中 配合getAppConfigFromRoutingContext(RoutingContext rc)使用
	 * 
	 * @param appConfig
	 * @param rc
	 */
	public static void setAppConfigToRoutingContext(AppConfig appConfig, RoutingContext rc) {
		rc.put(APP_CONFIG, appConfig);
	}

	/**
	 * 从路由上下文获取AppConfig
	 * 
	 * @param rc
	 * @return AppConfig
	 */
	public static AppConfig getAppConfigFromRoutingContext(RoutingContext rc) {
		return rc.get(APP_CONFIG);
	}

	/**
	 * 将AppConfig设置到上下文中 配合getApiConfigFromRoutingContext(RoutingContext rc)使用
	 * 
	 * @param apiConfig
	 * @param rc
	 */
	public static void setApiConfigIntoRoutingContext(ApiConfig apiConfig, RoutingContext rc) {
		rc.put(API_CONFIG, apiConfig);
	}

	/**
	 * 从路由上下文获取ApiConfig
	 * 
	 * @param rc
	 * @return ApiConfig
	 */
	public static ApiConfig getApiConfigFromRoutingContext(RoutingContext rc) {
		return rc.get(API_CONFIG);
	}

	public static boolean isEnableMockApi(ApiConfig apiConfig) {
		return apiConfig != null && apiConfig.getMockDefaultHttpStatus() != null;
	}

	/**
	 * 判断API服务类型是否是SAC规范服务类型
	 *
	 * @param apiServiceType
	 * @return
	 */
	public static boolean isSACServiceType(String apiServiceType) {
		return GatewayServiceType.SAC.getCode().equals(apiServiceType);
	}

	public static boolean isEnableDataSecurity(AppConfig appConfig) {
		return appConfig != null && appConfig.getDataSecurity() != null;
	}

	public static MockResponse mock(RoutingContext rc, ApiConfig apiConfig) {
		String apiServiceType = rc.get(API_SERVICE_TYPE);
		if (AppUtils.isEnableMockApi(apiConfig)) {
			// 如果是sac服务，query和body参数都从body中取
			Integer httpStatus = apiConfig.getMockDefaultHttpStatus();
			String mockResponse = apiConfig.getMockDefaultResponse();
			List<MockExpectation> mockExpectations = apiConfig.getMockExpectations();
			if (mockExpectations != null && !mockExpectations.isEmpty()) {
				for (MockExpectation mockExpectation : mockExpectations) {
					// 匹配条件需要全部匹配成功
					boolean matchSuccess = true;
					for (MockMatchCondition matchCondition : mockExpectation.getMatchConditions()) {
						if (!matchSuccess) {
							break;
						}
						MatchType matchType = MatchType.getByCode(matchCondition.getMatchType());
						if (matchType == null) {
							matchSuccess = false;
							break;
						}
						String parameterPosition = matchCondition.getParameterPosition();
						String parameterKey = matchCondition.getParameterKey();
						List<String> parameterValue = matchCondition.getParameterValue();
						String requestParameterValue = getRequestParameterValue(rc,
								AppUtils.isSACServiceType(apiServiceType), parameterPosition, parameterKey);
						if (!MatchType.IS_NULL.equals(matchType) && !MatchType.NOT_NULL.equals(matchType)) {
							// 需要值而没有设置值，直接匹配失败
							if (CollectionUtil.isEmpty(parameterValue) || requestParameterValue == null) {
								matchSuccess = false;
								break;
							}
						}

						switch (matchType) {
						case EQ:
							matchSuccess = parameterValue.get(0).equals(requestParameterValue);
							break;
						case NOT_EQ:
							matchSuccess = !parameterValue.get(0).equals(requestParameterValue);
							break;
						case GT:
							if (NumberUtil.isNumber(requestParameterValue)
									&& NumberUtil.isNumber(parameterValue.get(0))) {
								matchSuccess = new BigDecimal(requestParameterValue)
										.compareTo(new BigDecimal(parameterValue.get(0))) > 0;
							} else {
								matchSuccess = requestParameterValue.compareTo(parameterValue.get(0)) > 0;
							}
							break;
						case GE:
							if (NumberUtil.isNumber(requestParameterValue)
									&& NumberUtil.isNumber(parameterValue.get(0))) {
								matchSuccess = new BigDecimal(requestParameterValue)
										.compareTo(new BigDecimal(parameterValue.get(0))) >= 0;
							} else {
								matchSuccess = requestParameterValue.compareTo(parameterValue.get(0)) >= 0;
							}
							break;
						case LT:
							if (NumberUtil.isNumber(requestParameterValue)
									&& NumberUtil.isNumber(parameterValue.get(0))) {
								matchSuccess = new BigDecimal(requestParameterValue)
										.compareTo(new BigDecimal(parameterValue.get(0))) < 0;
							} else {
								matchSuccess = requestParameterValue.compareTo(parameterValue.get(0)) < 0;
							}
							break;
						case LE:
							if (NumberUtil.isNumber(requestParameterValue)
									&& NumberUtil.isNumber(parameterValue.get(0))) {
								matchSuccess = new BigDecimal(requestParameterValue)
										.compareTo(new BigDecimal(parameterValue.get(0))) <= 0;
							} else {
								matchSuccess = requestParameterValue.compareTo(parameterValue.get(0)) <= 0;
							}
							break;
						case IN:
							matchSuccess = parameterValue.contains(requestParameterValue);
							break;
						case NOT_IN:
							matchSuccess = !parameterValue.contains(requestParameterValue);
							break;
						case IS_NULL:
							matchSuccess = requestParameterValue == null;
							break;
						case NOT_NULL:
							matchSuccess = requestParameterValue != null;
							break;
						default:
							break;
						}

					}
					if (matchSuccess) {
						httpStatus = mockExpectation.getHttpStatus();
						mockResponse = mockExpectation.getMockResponse();
						break;
					}
				}
			}
			return new MockResponse(httpStatus, mockResponse);
		}
		return null;
	}
	
	/***
	 * BodyPostAnalysisHandler之后调用
	 * @param rc
	 * @param parameterKey
	 * @return
	 */
	private static String mockBodyVal(RoutingContext rc, String parameterKey) {
		String bodyStr = rc.get(GATEWAY_BODY_CODE);
		if(StringUtils.isBlank(bodyStr)) {
			throw new ServiceException(GatewayError.DEFAULT_SERVICE_ERROR);
		}
		JsonObject jsonBody = new JsonObject(bodyStr);
		return jsonBody.getString(parameterKey);
	}
	
	private static String getRequestParameterValue(RoutingContext rc, Boolean isServiceTypeSac,
			String parameterPosition, String parameterKey) {
		switch (parameterPosition) {
		case "query":
			if (isServiceTypeSac) {
				return mockBodyVal(rc, parameterKey);
			}
			return rc.request().getParam(parameterKey);
		case "header":
			return rc.request().getHeader(parameterKey);
		case "body":
			return mockBodyVal(rc, parameterKey);
		default:
			break;
		}
		return null;
	}

	public static String getResponseMsgByGatewayError(GatewayError gatewayError) {
		if (gatewayError != null) {
			return gatewayError.getReasonPhrase();
		}
		return getResponseMsgByGatewayError(GatewayError.DEFAULT_SERVICE_ERROR);
	}

	public static JsonObject getResponseJsonByGatewayError(GatewayError gatewayError) {
		if (gatewayError != null) {
			JsonObject jsonObject = new JsonObject();
			jsonObject.put("code", gatewayError.getCode());
			jsonObject.put("msg", gatewayError.getReasonPhrase());
			return jsonObject;
		}
		return getResponseJsonByGatewayError(GatewayError.DEFAULT_SERVICE_ERROR);
	}
	
    public static boolean isAnalysisBody(AppConfig appConfig, ApiConfig apiConfig, String contentType) {
        if (StringUtils.startsWith(contentType, "multipart")) {
            return false;
        }
        
        // uri不能模糊匹配
        if (StringUtils.equals(apiConfig.getUri(), "*")) {
            return false;
        }
        
        return isEnableDataSecurity(appConfig);
    }
}
